#ifndef GC_PARSER_H
#define GC_PARSER_H

#include <QList>
#include <parser.h>

#include "cacheprovider.h"

class GcParser : public QObject {
  Q_OBJECT;
  
 public:
  GcParser();
  ~GcParser();

  bool decodeOverview(const QByteArray&, CacheList &, bool);
  bool decodeInfo(const QByteArray&, Cache &);
  QString error() const;

 private:
  QJson::Parser m_parser;
  QString m_error;

  QString subString(QString &, const QString &, const QString &);

  // json decoder being run in two stages
  bool decodeJson(const QByteArray&, QMap<QString, QVariant>&);
  bool decodeStage1(const QByteArray&, QString &);
  bool decodeStage2(const QByteArray&, QMap<QString, QVariant>&);

  // overview specific content decoders
  bool parseOverviewRoot(const QMap<QString, QVariant> &, CacheList &, bool);
  bool parseOverviewCC(const QList<QVariant> &, CacheList &);
  bool parseOverviewEntry(const QMap<QString, QVariant> &, Cache &);

  // info specific content decoders
  bool parseInfoEntry(const QMap<QString, QVariant> &, Cache &);

  // generic content decoders
  bool parseFloat(const QMap<QString, QVariant> &, const QString &, qreal &);
  bool parseString(const QMap<QString, QVariant> &, const QString &, QString &);
  bool parseInt(const QMap<QString, QVariant> &, const QString &, int &);
  bool parseBool(const QMap<QString, QVariant> &, const QString &, bool &);
  bool parseCtid(const QMap<QString, QVariant> &, const QString &, Cache::Type &);
  bool parseList(const QMap<QString, QVariant> &, const QString &, QList<QVariant> &);
  bool parseMap(const QMap<QString, QVariant> &, const QString &, QMap<QString, QVariant> &);
  bool parseRating(const QMap<QString, QVariant> &, const QString &, Rating &);
  bool parseContainer(const QMap<QString, QVariant> &, const QString &, Container &);
  bool parseDate(const QMap<QString, QVariant> &, const QString &, QDate &);
};

#endif // GC_PARSER_H
